==> blcr-0.2.0.tar.gz: blcr-0.2.0/vmadump/vmadump.h <==
/*-------------------------------------------------------------------------
 *  vmadump.h:  Definitions for VMADump
 *
 *  Copyright (C) 1999-2001 by Erik Hendriks <erik@hendriks.cx>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: vmadump.h,v 1.11 2003/09/04 01:35:19 eroman Exp $
 *-----------------------------------------------------------------------*/
#ifndef _VMADUMP_H
#define _VMADUMP_H

/* Everything past this struct is architecture dependent */
struct vmadump_header {
    unsigned char magic[3];
    unsigned char fmt_vers;	/* version of dump format */
    unsigned char arch;
    unsigned char major;	/* Kernel rev dumped from */
    unsigned char minor;
    unsigned char patch;
};

struct vmadump_vma_header {
    unsigned long  start;	/* ~0 = end of list */
    unsigned long  end;
    unsigned short flags;
    unsigned short namelen;	/* 0 = data follows */
    unsigned long  offset;	/* file offset for mmap */
};

struct vmadump_page_header {
    unsigned long start;	/* ~0 = end of list */
};

struct vmadump_mm_info {
    unsigned long start_code, end_code;
    unsigned long start_data, end_data;
    unsigned long start_brk,  brk;
    unsigned long start_stack;
    unsigned long arg_start, arg_end;
    unsigned long env_start, env_end;
};

#define VMAD_MAGIC {0x56,0x4d,0x41}
#define VMAD_FMT_VERS 6

#define VMAD_ARCH_i386  1
#define VMAD_ARCH_sparc 2
#define VMAD_ARCH_alpha 3
#define VMAD_ARCH_ppc   4

#if defined(__i386__)
#define VMAD_ARCH VMAD_ARCH_i386
#define HAVE_BINFMT_VMADUMP 1
#define __NR_vmadump 209
#elif defined(__sparc__)
#define VMAD_ARCH VMAD_ARCH_sparc
#define HAVE_BINFMT_VMADUMP 1
/*#define __NR_vmadump 209*/
#elif defined(__alpha__)
#define VMAD_ARCH VMAD_ARCH_alpha
/* Alpha execve doesn't save enough state on the way into the kernel
 * to support complete state restoration, so no binfmt_vmadump on
 * alpha. */
#undef HAVE_BINFMT_VMADUMP
/*#define __NR_vmadump 290*/
#elif defined(powerpc)
#define VMAD_ARCH VMAD_ARCH_ppc
#define HAVE_BINFMT_VMADUMP 1
/*#define __NR_vmadump 223*/
#else
#error VMADUMP does not support this architecture 
#endif

/* Defs for the syscall interface */
#define VMAD_DO_DUMP       0x00 
#define VMAD_DO_UNDUMP     0x01
#define VMAD_DO_EXECDUMP   0x02
#define VMAD_DO_THREADDUMP 0x03
#define VMAD_DO_THREADUNDUMP 0x04

#define VMAD_LIB_CLEAR 0x30
#define VMAD_LIB_ADD   0x31
#define VMAD_LIB_DEL   0x32
#define VMAD_LIB_LIST  0x33
#define VMAD_LIB_SIZE  0x34

/* Flags for dump/undump */
#define VMAD_DUMP_LIBS  1
#define VMAD_DUMP_EXEC  2
#define VMAD_DUMP_OTHER 4
#define VMAD_DUMP_ALL   7
#define VMAD_DUMP_REGSONLY 8

#define VMAD_THREAD_DUMP   0x01
#define VMAD_THREAD_UNDUMP 0x10

struct vmadump_execdump_args {
    const char *arg0;
    char * const *argv;
    char * const *envp;
    
    int fd;
    int flags;
};

struct vmadump_threaddump_args {
    int num_threads;
    int fd;
    int flags;
};

struct vmadump_threadundump_args {
    int num_threads;                    
    int fd;
    int flags;
    int old_pid;                        /* OUT: Old PID of restored process */
};

#ifdef __KERNEL__
struct vmadump_thread_req {
    struct mm_struct *mm;               /* which mm this req corresponds to */
    int num_threads;                    /* tot# of threads for this dump */
    atomic_t num_awake;                 /* # of threads not yet checked in */
    wait_queue_head_t thread_barrier;   /* wait for dump to happen */
    atomic_t num_frozen;                /* # of threads that have not frozen yet (exit) */
    wait_queue_head_t exit_barrier;     /* wait for everyone to finish before we exit */
    struct file *file;                  /* file to dump to */
    struct semaphore file_sem;          /* mutex to protect file */
    int flags;                          /* stuff from user space */
    atomic_t ref_count;                 /* # of threads ref'ing this */
    struct list_head req_list;          /* for thread_req_list */
    int type;
    char comm[16];                      /* name of the application being restored */
};

struct vmadump_map_ctx {
    /* map_open:
     *  given a file, open it.  The file should be opened with
     *  O_RDONLY.  The filename argument is a kernel space pointer.
     *  The semantics should be identical to:
     *    filp_open(filename, O_RDONLY);
     *
     * map_name:
     *  given a file structure, return the canonical name for it.  The
     *  semantics should be identical to:
     *    d_path(file->f_dentry, file->f_vfsmnt, buffer, size);
     *
     * read_file, write_file:
     *  Alternate file read/write routines.  These should have the
     *  same semantics as a blocking sys_read and sys_write.  */
    struct file *(*map_open)  (struct vmadump_map_ctx *ctx,
			       const char *filename);
    char        *(*map_name)  (struct vmadump_map_ctx *ctx, struct file *,
			       char *buffer, int size);
    ssize_t      (*read_file) (struct vmadump_map_ctx *, struct file *,
			       void *, size_t);
    ssize_t      (*write_file)(struct vmadump_map_ctx *, struct file *,
			       const void *, size_t);
    void *private;
};
extern long vmadump_freeze_proc(struct vmadump_map_ctx *, struct file *file,
				struct pt_regs *regs, int flags, 
				int num_threads);
extern long vmadump_thaw_proc  (struct vmadump_map_ctx *, struct file *file,
				int flags, int *pid, struct pt_regs *regs);
extern long vmadump_freeze_threads(struct file *file, struct pt_regs *regs,
                                   struct vmadump_threaddump_args *args);
extern long vmadump_thaw_threads(struct file *file, struct pt_regs *regs,
                                 struct vmadump_threadundump_args *args);
extern long do_vmadump         (struct pt_regs *regs);
#endif /* ifdef __KERNEL__ */
#endif

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */

==> blcr-0.5.0.tar.gz: blcr-0.5.0/vmadump/vmadump.h <==
/*-------------------------------------------------------------------------
 *  vmadump.h:  Definitions for VMADump
 *
 *  Copyright (C) 1999-2001 by Erik Hendriks <erik@hendriks.cx>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: vmadump.h,v 1.16 2007/02/22 21:46:19 phargrov Exp $
 *-----------------------------------------------------------------------*/
#ifndef _VMADUMP_H
#define _VMADUMP_H

/* Everything past this struct is architecture dependent */
struct vmadump_header {
    unsigned char magic[3];
    unsigned char fmt_vers;	/* version of dump format */
    unsigned char arch;
    unsigned char major;	/* Kernel rev dumped from */
    unsigned char minor;
    unsigned char patch;
};

struct vmadump_vma_header {
    unsigned long  start;	/* ~0 = end of list */
    unsigned long  end;
    unsigned short flags;
    unsigned short namelen;	/* 0 = data follows */
    unsigned long  offset;	/* file offset for mmap */
};

struct vmadump_page_header {
    unsigned long start;	/* ~0 = end of list */
    unsigned int num_pages;
};

struct vmadump_mm_info {
    unsigned long start_code, end_code;
    unsigned long start_data, end_data;
    unsigned long start_brk,  brk;
    unsigned long start_stack;
    unsigned long arg_start, arg_end;
    unsigned long env_start, env_end;
};

#define VMAD_MAGIC {0x56,0x4d,0x41}
#define VMAD_FMT_VERS 6

#define VMAD_ARCH_i386  1
#define VMAD_ARCH_sparc 2
#define VMAD_ARCH_alpha 3
#define VMAD_ARCH_ppc   4

#if defined(__i386__)
#define VMAD_ARCH VMAD_ARCH_i386
#define HAVE_BINFMT_VMADUMP 1
#define __NR_vmadump 209
#elif defined(__sparc__)
#define VMAD_ARCH VMAD_ARCH_sparc
#define HAVE_BINFMT_VMADUMP 1
/*#define __NR_vmadump 209*/
#elif defined(__alpha__)
#define VMAD_ARCH VMAD_ARCH_alpha
/* Alpha execve doesn't save enough state on the way into the kernel
 * to support complete state restoration, so no binfmt_vmadump on
 * alpha. */
#undef HAVE_BINFMT_VMADUMP
/*#define __NR_vmadump 290*/
#elif defined(powerpc)
#define VMAD_ARCH VMAD_ARCH_ppc
#define HAVE_BINFMT_VMADUMP 1
/*#define __NR_vmadump 223*/
#else
#error VMADUMP does not support this architecture 
#endif

/* Defs for the syscall interface */
#define VMAD_DO_DUMP       0x00 
#define VMAD_DO_UNDUMP     0x01
#define VMAD_DO_EXECDUMP   0x02
#define VMAD_DO_THREADDUMP 0x03
#define VMAD_DO_THREADUNDUMP 0x04

#define VMAD_LIB_CLEAR 0x30
#define VMAD_LIB_ADD   0x31
#define VMAD_LIB_DEL   0x32
#define VMAD_LIB_LIST  0x33
#define VMAD_LIB_SIZE  0x34

/* Flags for dump/undump */
#define VMAD_DUMP_LIBS  1
#define VMAD_DUMP_EXEC  2
#define VMAD_DUMP_OTHER 4
#define VMAD_DUMP_ALL   7
#define VMAD_DUMP_REGSONLY 8

#define VMAD_THREAD_DUMP   0x01
#define VMAD_THREAD_UNDUMP 0x10

struct vmadump_execdump_args {
    const char *arg0;
    char * const *argv;
    char * const *envp;
    
    int fd;
    int flags;
};

struct vmadump_threaddump_args {
    int num_threads;
    int fd;
    int flags;
};

struct vmadump_threadundump_args {
    int num_threads;                    
    int fd;
    int flags;
    int old_pid;                        /* OUT: Old PID of restored process */
};

#ifdef __KERNEL__
struct vmadump_thread_req {
    struct mm_struct *mm;               /* which mm this req corresponds to */
    int num_threads;                    /* tot# of threads for this dump */
    atomic_t num_awake;                 /* # of threads not yet checked in */
    atomic_t num_frozen;                /* # of threads that have not frozen yet (exit) */
    wait_queue_head_t wait;   		/* wait for dump to happen */
    struct file *file;                  /* file to dump to */
    struct semaphore sema;              /* mutex to serialize output */
    int done_leader;                    /* ensure full freeze/thaw done by 1 thread */
    int flags;                          /* stuff from user space */
    atomic_t ref_count;                 /* # of threads ref'ing this */
    struct list_head req_list;          /* for thread_req_list */
    int type;
    char comm[16];                      /* name of the application being restored */
    int error;
};

struct vmadump_map_ctx {
    /* map_open:
     *  given a file, open it.  The file should be opened with
     *  the given flags.  The filename argument is a kernel space pointer.
     *  The semantics should be identical to:
     *    filp_open(filename, flags);
     *
     * map_name:
     *  given a file structure, return the canonical name for it.  The
     *  semantics should be identical to:
     *    d_path(file->f_dentry, file->f_vfsmnt, buffer, size);
     *
     * read_file, write_file:
     *  Alternate file read/write routines.  These should have the
     *  same semantics as a blocking sys_read and sys_write.  */
    struct file *(*map_open)  (struct vmadump_map_ctx *ctx,
			       const char *filename, int flags);
    char        *(*map_name)  (struct vmadump_map_ctx *ctx, struct file *,
			       char *buffer, int size);
    ssize_t      (*read_file) (struct vmadump_map_ctx *, struct file *,
			       void *, size_t);
    ssize_t      (*write_file)(struct vmadump_map_ctx *, struct file *,
			       const void *, size_t);
    void *private;
};
extern long vmadump_freeze_proc(struct vmadump_map_ctx *, struct file *file,
				struct pt_regs *regs, int flags, 
				int num_threads);
extern long vmadump_thaw_proc  (struct vmadump_map_ctx *, struct file *file,
				struct pt_regs *regs, int flags, int *pid_p);
extern long vmadump_freeze_threads(struct file *file, struct pt_regs *regs,
                                   struct vmadump_threaddump_args *args);
extern long vmadump_thaw_threads(struct file *file, struct pt_regs *regs,
                                 struct vmadump_threadundump_args *args);
extern long do_vmadump         (struct pt_regs *regs);
#endif /* ifdef __KERNEL__ */
#endif

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
